<?php

require_once "prepend.php";
require_once "XEmb_Config.class.php";

class XEmb_Cache
{
  var $cache = array();
  var $http_client;
  var $cache_lite = NULL;

  /**
   * Permet de recuperer une instance de notre classe
   */
  function &Instance()
  {
    static $instance;
    
    if (!isset($instance))
      $instance = new XEmb_Cache();
    return $instance;
  }

  function XEmb_Cache()
  {
    require_once ('Cache/Lite.php');
    $options["cacheDir"] = dirname(__FILE__)."/../../../cache/";
    $options["lifeTime"] = 60*60*24;
    $options["caching"] = true;
    $this->cache_lite = new Cache_Lite($options);
  }

  /**
   * Lit un fichier  partir de son url
   * Si le fichier  deja t charg, il le recupere dans le cache
   */
  function &loadFile( $url )
  {
    // on regarde dans le cache
    $file_content =& $this->getData($url, "file");
    if ($file_content == NULL)
      {
        //echo "ON CHARGEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE";
        // le fichier n'est dans aucun cache, on le recharge
        if (preg_match("/http:\/\/.*/i", $url, $matches))
          {
            // fichier distant
            $client =& $this->_getHTTP_Client();
            
            // on recupere le contenu du fichier
            $client->get($url);
            $reponse =& $client->currentResponse();
            
            $this->setData( $url, "file-date", $reponse["headers"]["Last-Modified"] );
            $this->setData( $url, "file", $reponse["body"] );
          }
        else
          {
            // fichier local
            $this->setData( $url, "file", file_get_contents($url));
          }
      }
    return $this->getData( $url, "file" );
  }

  function &getData( $url, $type )
  {

    // on commence par regarder dans le cache memoire
    $data =& $this->cache[$url][$type];
    if (isset($data))
      {
        //        echo "url=".$url." type=".$type." data=".$data."\n";
        return $data;
      }

    // si le cache n'est pas activ dans les options, on ne cherche pas a charger le fichier depuis le cache
    if ( !XEmb_Config::GetOption("cache") )
      return NULL;
    
    // on regarde dans le cache lite (filesysteme)
    $data =& $this->cache_lite->get($url."_".$type);
    //    print_r($data);
    if ($data != NULL && $type != "file-date")
      {
        //        print_r("CHECK DATEEEEEEEEEEEEEE");
        // ok, on a trouve quelque chose, on verifie si la date est bonne
        $last_date =& $this->getData($url, "file-date");
        //        print_r($last_date);
        if ($last_date != $this->_getFileDate($url))
          {
            //            echo "MODIFIED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!";
            // on vide l'ancienne page du cache
            $this->cache_lite->remove($url."_file");
            $this->cache_lite->remove($url."_content");
            $this->cache_lite->remove($url."_meta");
            $this->cache_lite->remove($url."_title");
            return NULL;
          }
      }
    //echo "url=".$url." type=".$type." data=".$data."\n";
    return $data;
  }

  function setData( $url, $type, &$data )
  {
    $this->cache[$url][$type] =& $data;

    // si le cache n'est pas activ dans les options, on ne cherche pas a charger le fichier depuis le cache
    if ( !XEmb_Config::GetOption("cache") )
      return NULL;

    $this->cache_lite->save($data, $url."_".$type);
  }

  /*
   * Cree un client HTTP
   */
  function &_getHTTP_Client()
  {
    if (!isset($this->http_client))
      {
        require_once 'HTTP/Client.php';
        $client =& new HTTP_Client();
        //    $client->setRequestParameter("proxy_host", "proxy.free.fr");
        //    $client->setRequestParameter("proxy_port", "3128");
        $client->setDefaultHeader("User-Agent",
                                  XEmb_Config::GetAppName()." v".
                                  XEmb_Config::GetAppVersion()." par ".
                                  XEmb_Config::GetAppAuteur()." (".
                                  XEmb_Config::GetAppHelpURL().")");
        $this->http_client =& $client;
      }
    return $this->http_client;
  }

  function _getFileDate($url)
  {
    if (preg_match("/http:\/\/.*/i", $url, $matches))
      {
        $client =& $this->_getHTTP_Client();
        $client->head($url);
        $reponse =& $client->currentResponse();
        if (!isset($reponse["headers"]["Last-Modified"]))
          $reponse["headers"]["Last-Modified"] = date("D, j M Y G:i:s T");
        $this->setData( $url, "file-date", $reponse["headers"]["Last-Modified"] );
        return $reponse["headers"]["Last-Modified"];
      }
    else
      return "";
  }
  
  function clean()
  {
    $this->cache_lite->clean();
  }
};

?>