<?php

require_once "prepend.php";
require_once "XEmb_Cache.class.php";
require_once "XEmb_Config.class.php";

class XEmb_Transformations
{
  var $pattern = "/{[Xx][Ee]mb url=\"([a-zA-Z0-9\:\.\/_-~]+)\"( *meta=\"(yes|no)\"|)( *chapo=\"(yes|no)\"|)}/i";
  
  /**
   * Permet de recuperer une instance de notre classe
   */
  function &Instance()
  {
    static $instance;
    
    if (!isset($instance))
      $instance = new XEmb_Transformations();
    return $instance;
  }

  /**
   * Cherche si le contenu contient le motif identifiant un document externe (XEmb)
   */
  function searchPatternMatch( &$content )
  {
    $result = array();
    if (!preg_match($this->pattern, $content, $matches))
      $result["ok"] = false;
    else
      {
        //        print_r($matches);
        $result["ok"] = true;
        $result["url"]   = $matches[1];
        $result["meta"] = isset($matches[3]) ? $matches[3]: (XEmb_Config::GetOption("meta_default") ? "yes" : "no");
        $result["chapo"] = isset($chapo[5]) ? $matches[5]: "no";
      }
    return $result;
  }

  
  /**
   * Retourne le contenu du document externe (entre <body> et </body>)
   * Les url des attributs "href" et "src" seront transformes.
   */
  function &getContent( $url )
  {
    $cache =& XEmb_Cache::Instance();
    $content =& $cache->getData($url, "content");
    if ($content == NULL)
      {
      $parameters["article_path"] = substr($url,0,strrpos($url,"/")+1);
      // load xhtml documents and xslt files
      $article_content      = $cache->loadFile($url);
      $xslt_extract_content = $cache->loadFile(dirname(__FILE__)."/extract_content.xsl");
      // extract informations from XHTML document
      $arguments["/_xml"] = $article_content;
      $xh = xslt_create();
      $arguments["/_xsl"] = $xslt_extract_content;
      xslt_set_encoding($xh, dc_encoding);
      $content =& xslt_process($xh, 'arg:/_xml', 'arg:/_xsl', NULL, $arguments, $parameters);
      xslt_free($xh);

      // on stoque ce que l'on a calcul dans le cache
      $cache->setData($url, "content", $content);
    }
    return $content;    
  }

  /**
   * Retourne le titre du document externe (entre <title> et </title>)
   */
  function &getTitle( $url )
  {
    $cache =& XEmb_Cache::Instance();
    $title =& $cache->getData($url, "title");
    if ($title == NULL)
    {
      $parameters["article_path"] = substr($url,0,strrpos($url,"/")+1);
      // load xhtml documents and xslt files
      $article_content    = $cache->loadFile($url);
      $xslt_extract_title = $cache->loadFile(dirname(__FILE__)."/extract_title.xsl");
      // extract informations from XHTML document
      $arguments["/_xml"] = $article_content;
      $xh = xslt_create();
      $arguments["/_xsl"] = $xslt_extract_title;
      xslt_set_encoding($xh, dc_encoding);
      $title =& xslt_process($xh, 'arg:/_xml', 'arg:/_xsl', NULL, $arguments, $parameters);
      xslt_free($xh);

      // on stoque ce que l'on a calcul dans le cache
      $cache->setData($url, "title", $title);
    }
    return $title;
  }

  /**
   * Retourne les meta donnes document externe (les feuilles de style)
   * Les url seront transformes.
   */
  function getMeta( $url )
  {
    $cache =& XEmb_Cache::Instance();
    $meta =& $cache->getData($url, "meta");
    if ($meta == NULL)
      {
      $parameters["article_path"] = substr($url,0,strrpos($url,"/")+1);
      // load xhtml documents and xslt files
      $article_content   = $cache->loadFile($url);
      $xslt_extract_meta = $cache->loadFile(dirname(__FILE__)."/extract_meta.xsl");
      // extract informations from XHTML document
      $arguments["/_xml"] = $article_content;
      $xh = xslt_create();
      $arguments["/_xsl"] = $xslt_extract_meta;
      xslt_set_encoding($xh, dc_encoding);
      $meta =& xslt_process($xh, 'arg:/_xml', 'arg:/_xsl', NULL, $arguments, $parameters);
      xslt_free($xh);

      // on stoque ce que l'on a calcul dans le cache
      $cache->setData($url, "meta", $meta);
    }
    return $meta;
  }
};

?>